#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   lib.sh of /CoreOS/authconfig/Library/basic
#   Description: Basic library for setting up authconfig environment
#   Authors: David Spurek <dspurek@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2013 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   library-prefix = authconfig
#   library-version = 3
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

: <<'=cut'
=pod

=head1 NAME

/CoreOS/authconfig/Library/basic - Basic library for authconfig testing

=head1 DESCRIPTION

This is up-to-date authconfig library which provides various helper functions
for testing authconfig.

USAGE:
To setup phase paste: authconfig_setup
To cleanup phase paste: authconfig_cleanup

=cut

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Variables
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

: <<'=cut'
=pod

=head1 VARIABLES

Below is the list of global variables. When writing a new library,
please make sure that all global variables start with the library
prefix to prevent collisions with other libraries.

=over

=item authconfig_backedup_files

Default list of variables that should be backed up because authconfig
can modify these files. They are backed up with --clean parameter because they don't have to always exist in a system.

=item authconfig_destructive_backedup_files

Default list of variables that should be backed up because authconfig
can modify these files. These files are backed up without --clean parameter because they are always exist in the system and their deletion may be destructive.

=item authconfig_services

List of services that may be affected by authconfig.

=back

=cut


authconfig_backedup_files="
/etc/group
/etc/gshadow
/etc/nsswitch.conf
/etc/pam.d/password-auth
/etc/pam.d/password-auth-ac
/etc/pam.d/system-auth
/etc/pam.d/system-auth-ac
/etc/passwd
/etc/shadow
/etc/dconf
/etc/hesiod.conf
/etc/krb.conf
/etc/krb5.conf
/etc/ldap.conf
/etc/libuser.conf
/etc/login.defs
/etc/nslcd.conf
/etc/nss_ldap.conf
/etc/openldap/ldap.conf
/etc/openldap/slapd.conf
/etc/openldap/cacerts/
/etc/pam_ldap.conf
/etc/pam_pkcs11/pam_pkcs11.conf
/etc/pam_pkcs11/pkcs11_eventmgr.conf
/etc/pam_pkcs11/pam_pkcs11.conf
/etc/pam_pkcs11/pkcs11_eventmgr.conf
/etc/pam.d/fingerprint-auth
/etc/pam.d/fingerprint-auth-ac
/etc/pam.d/postlogin
/etc/pam.d/postlogin-ac
/etc/pam.d/smartcard-auth
/etc/pam.d/smartcard-auth-ac
/etc/pam_smb.conf
/etc/samba/smb.conf
/etc/security/pwquality.conf
/etc/sssd/sssd.conf
/var/lib/sss/
/etc/sysconfig/authconfig
/etc/sysconfig/network
/etc/yp.conf
/var/db/nscd
"

authconfig_services="sssd nslcd winbind ypbind oddjobd nscd"

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Functions
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

: <<'=cut'
=pod

=head1 FUNCTIONS

=head2 authconfig_setup

Function backup files that may auhtconfig change and backup state of enabled|disabled services.
Backups actual state of selinux booleans too.

=cut


function authconfig_setup ()
{
    # Backup the current state of all the SELinux booleans.
    rlRun "rlSEBooleanBackup" 0,33
    # backup files that could be changed by authconfig
    rlRun "rlFileBackup --namespace authconfiglib --clean ${authconfig_backedup_files//$'\n'/ }" 0 "Backup files that could be changed by authconfig"
    # backup nscd service state
    rpm -q nscd && rlServiceStop nscd
    #save current state of enabled|disabled services
    rlRun "authconfig_save_services_runlevels"

    # remove /etc/sysconfig/authconfig file, just for sure
    # it shouldn't exist by default
    if [[ -e /etc/sysconfig/authconfig ]]; then
        rlLog "flush  /etc/sysconfig/authconfig file,it should be empty by default"
        rlRun "> /etc/sysconfig/authconfig"
    fi
}

: <<'=cut'
=pod

=head2 authconfig_cleanup

Function restores backed up files, state of enabled|disabled services and selinux booleans.

=cut

function authconfig_cleanup ()
{
    #restore state of enabled|disabled services
    rlRun "authconfig_restore_services_runlevels"
    rpm -q nscd && rlServiceRestore nscd
    rlRun "rlFileRestore --namespace authconfiglib" 0 "Restore backed up files"
    rlRun "rlSEBooleanRestore"
}

: <<'=cut'
=pod

=head2 authconfig_save_services_runlevels

Function saves current state of enabled|disabled services.

=cut

function authconfig_save_services_runlevels ()
{
    rlLog "Saving services runlevel values"
    if rlIsRHEL '<7'; then
        chkconfig --list > chk_begin.out
    else
        systemctl list-unit-files > chk_begin.out
    fi
}

: <<'=cut'
=pod

=head2 authconfig_restore_services_runlevels

Function restores state of enabled|disabled services.

=cut

function authconfig_restore_services_runlevels ()
{
    rlLog "Restoring services runlevel values"
    rm -f chk_changes.out
    touch chk_changes.out
    if rlIsRHEL '<7'; then
        chkconfig --list > chk_end.out
        # get changed service and their original runlevel values in form 'service on|off'
        diff chk_begin.out chk_end.out | grep "^<" | awk '{print $2 " " $6}'| tr -d ":" | tr -d '[:digit:]' > chk_changes_all.out
        # filter list of changed services only to services that may authconfig change
        for i in $authconfig_services; do
            cat chk_changes_all.out | grep $i >> chk_changes.out
        done
        while read line; do
            service_name=`echo -e "$line" | awk '{print $1}'`
            service_value=`echo -e "$line" | awk '{print $2}'`
            rlLog "Changing $service_name to $service_value"
            chkconfig --level 345 $service_name $service_value
        done < chk_changes.out
    else
        systemctl list-unit-files > chk_end.out
        diff chk_begin.out chk_end.out | grep "^<" | grep service > chk_changes_all.out
        # filter list of changed services only to services that may authconfig change
        for i in $authconfig_services; do
            cat chk_changes_all.out | grep $i >> chk_changes.out
        done
        while read line; do
            service_name=`echo -e "$line" | awk '{print $2}'`
            # sed 's/.$//' removes last character
            service_value=`echo -e "$line" | awk '{print $3}' | sed 's/.$//'`
            rlLog "Changing $service_name to $service_value"
            systemctl $service_value $service_name
        done < chk_changes.out
    fi
    return 0
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Verification
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

authconfigLibraryLoaded() {
    for PKG in authconfig; do
        if rpm=$(rpm -q $PKG); then
            rlLogDebug "Library authconfig/basic running with $rpm"
        else
            if rlIsRHEL '<7'; then
                rlLogError "Package $PKG not installed"
            else
                rlLog "Package $PKG not installed."
            fi
        fi
    done
    return 0
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Authors
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

: <<'=cut'
=pod

=head1 AUTHORS

=over

=item *

David Spurek <dspurek@redhat.com>

=back

=cut
